<?php

namespace Drupal\beehotel_pricealterator;

use Drupal\Component\Plugin\Factory\DefaultFactory;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\beehotel_pricealterator\Annotation\PriceAlterator;
use Drupal\Core\Session\AccountInterface;


/**
 * A plugin manager for pricealterator plugins.
 *
 * The PriceAlteratorPluginManager class extends the DefaultPluginManager to provide
 * a way to manage pricealterator plugins. A plugin manager defines a new plugin type
 * and how instances of any plugin of that type will be discovered, instantiated
 * and more.
 *
 * Using the DefaultPluginManager as a starting point sets up our pricealterator
 * plugin type to use annotated discovery.
 *
 * The plugin manager is also declared as a service in
 * beehotel_pricealterator.services.yml so that it can be easily accessed and used
 * anytime we need to work with pricealterator plugins.
 */
class PriceAlteratorPluginManager extends DefaultPluginManager {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;


  /**
   * Creates the discovery object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   *
   *
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler, AccountInterface $current_user) {
    // We replace the $subdir parameter with our own value.
    // This tells the plugin manager to look for PriceAlterator plugins in the
    // 'src/Plugin/PriceAlterator' subdirectory of any enabled modules. This also
    // serves to define the PSR-4 subnamespace in which pricealterator plugins will
    // live. Modules can put a plugin class in their own namespace such as
    // Drupal\{module_name}\Plugin\PriceAlterator\MyPriceAlteratorPlugin.
    $subdir = 'Plugin/PriceAlterator';

    // The name of the interface that plugins should adhere to. Drupal will
    // enforce this as a requirement. If a plugin does not implement this
    // interface, Drupal will throw an error.
    $plugin_interface = PriceAlteratorInterface::class;

    // The name of the annotation class that contains the plugin definition.
    $plugin_definition_annotation_name = PriceAlterator::class;

    parent::__construct($subdir, $namespaces, $module_handler, $plugin_interface, $plugin_definition_annotation_name);

    // This allows the plugin definitions to be altered by an alter hook. The
    // parameter defines the name of the hook, thus: hook_pricealterator_info_alter().
    // We implement this hook to change the plugin definitions:
    // see beehotel_pricealterator_pricealterator_info_alter().
    $this->alterInfo('pricealterator_info');

    // This sets the caching method for our plugin definitions. Plugin
    // definitions are discovered by examining the $subdir defined above, for
    // any classes with an $plugin_definition_annotation_name. The annotations
    // are read, and then the resulting data is cached using the provided cache
    // backend. For our PriceAlterator plugin type, we've specified the @cache.default
    // service be used in the beehotel_pricealterator.services.yml file. The second
    // argument is a cache key prefix. Out of the box Drupal with the default
    // cache backend setup will store our plugin definition in the cache_default
    // table using the pricealterator_info key. All that is implementation details
    // however, all we care about it that caching for our plugin definition is
    // taken care of by this call.
    $this->setCacheBackend($cache_backend, 'pricealterator_info', ['pricealterator_info']);
    $this->currentUser = $current_user;

  }

  /**
   * Gets alterators sorted by weight.
   *
   * @return array
   *   Returns an array of all alterators
   */
  public function SortedAlterators() {

    $alterators = $this->getDefinitions();
    $alterators = $this->WeightValidation($alterators);
    array_multisort(array_column($alterators, 'weight'), SORT_ASC, $alterators);
    return $alterators;

  }

  /**
   * Alterators from custom modules are only allowed with
   * weight 1xxx (from 1000 till 1999)
   *
   * @return array
   *   Returns an array of weight-validated alterators
   */
  public function WeightValidation($alterators) {

    //@TODO: implement validation
    return $alterators;

  }

}
